
=head1 NAME

model4.pm

=head1 SYNOPSIS

Statistical library package to calculate the Loglikelihood ratio for 
4grams. This package should be used with statistic.pl and rank.pl.

=head1 DESCRIPTION

The log-likelihood ratio measures the devitation between the observed data 
and what would be expected if <word1>, <word2>, <word3> and <word4>  were  
independent. The higher the score, the less evidence there is in favor of  
concluding that the words are independent. 

=head1 AUTHORS

Ted Pedersen <tpederse@d.umn.edu>

Bridget Thomson McInnes <bthomson@d.umn.edu>

=head1 BUGS

This measure currently only defined for 4gram data stored in 4x4x4x4
contingency table. 

=head1 SEE ALSO

Mailing List: http://groups.yahoo.com/ngram

=head1 COPYRIGHT

Copyright 2000-2004 by Ted Pedersen and Satanjeev Banerjee and Amruta 
Purandare

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=cut

package model4;

#  Make sure that ll4model.pm is available in the PATH. First
#  we check in the directory you are running from, and then we
#  look through the system path. If the module is not found anywhere
#  then abort. 

my $module = "ll4model.pm"; my $modulename = "ll4model.pm";

if( !( -f $modulename ) ) {
    my $found = 0;
    #  Check each of the PATHS to see if the module is there
    foreach (split(/:/, $ENV{PATH})) {
 	$module = $_ . "/" . $modulename;
	if ( -f $module ) { $found = 1; last; }
    }
    # if still not found anywhere, quit!
    if ( ! $found ) { print "Could not find $modulename.\n"; exit; }
}

# Include the module into the current package.    

require $module;
require Exporter;

@ISA = qw ( Exporter );
@EXPORT = qw (initializeStatistic getStatisticName calculateStatistic errorCode errorString);

# function to set up various variables before the actual computation
# starts. also to check if we are being given bigrams, and if our
# frequency combinations are enough to do the computation
sub initializeStatistic
{    
    ll4model::initializeStatistic(@_);
}

# function to calculate the ll value!
sub calculateStatistic
{
    #  The parameters passed into calculateStatistic
    #  need to be passed into checkObservedValues for it 
    #  to calculate the Observed values. If there is a 
    #  problem with the observed values, this function 
    #  will return, causing the measure to abort its 
    #  computations and return with a zero code.
    
    if( !( $check = ll4model::checkObservedValues(@_) ) ) {
	return(0);
    }
    
    #  Check the expected values. Also, the quotient of 
    #  the observed and expected value should not be less 
    #  than zero. If there is a problem with the expected 
    #  values, this function will return, causing the measure 
    #  to abort and return with a zero code.
    
    if( !( $check = ll4model::checkExpectedValues() ) ) {
	return(0);
    }
    
    #  Now we can retrieve the log likelihood scores for each
    #  of the fourteen models
    
    if( ! ( ($LogLikelihood1,  $LogLikelihood2,  $LogLikelihood3,  $LogLikelihood4, 
	     $LogLikelihood5,  $LogLikelihood6,  $LogLikelihood7,  $LogLikelihood8,  
	     $LogLikelihood9,  $LogLikelihood10, $LogLikelihood11, $LogLikelihood12, 
	     $LogLikelihood13, $LogLikelihood14) = ll4model::getLogLikelihoodValues())) {
	return(0);
    }
    
    #  Find the model that the ngram 'fits' the best and return that score
    my $score = $LogLikelihood1; my $model = 1;
    if($score > $LogLikelihood2) { $score = $LogLikelihood2; $model = 2; }
    if($score > $LogLikelihood3) { $score = $LogLikelihood3; $model = 3; }
    if($score > $LogLikelihood4) { $score = $LogLikelihood4; $model = 4; }
    if($score > $LogLikelihood5) { $score = $LogLikelihood5; $model = 5; }
    if($score > $LogLikelihood6) { $score = $LogLikelihood6; $model = 6; }
    if($score > $LogLikelihood7) { $score = $LogLikelihood7; $model = 7; }
    if($score > $LogLikelihood8) { $score = $LogLikelihood8; $model = 8; }
    if($score > $LogLikelihood9) { $score = $LogLikelihood9; $model = 9; }
    if($score > $LogLikelihood10) { $score = $LogLikelihood10; $model = 10; }
    if($score > $LogLikelihood11) { $score = $LogLikelihood11; $model = 11; }
    if($score > $LogLikelihood12) { $score = $LogLikelihood12; $model = 12; }
    if($score > $LogLikelihood13) { $score = $LogLikelihood13; $model = 13; }
    if($score > $LogLikelihood14) { $score = $LogLikelihood14; $model = 14; }
    
    return $score;
    
}


# function to return the error code of the last operation and reset
# error code. useful if the error can be recovered from!
sub errorCode 
{ 
    return ll4model::errorCode();
}

# function to return the error message of the last operation and reset
# the message string. useful if error can be recovered from!
sub errorString
{
    return ll4model::errorString();
}

# function to return the name of this statistic
sub getStatisticName
{
    return "Loglikelihood4";
}

1;

